// Description:
//   Wrapper for GL_ARB_texture_cube_map extension.
//
// Copyright (C) 2001 Frank Becker
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation;  either version 2 of the License,  or (at your option) any  later
// version.
//
// This program is distributed in the hope that it will be useful,  but  WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
//
#include <GLVertexArrayEXT.hpp>

PFNGLARRAYELEMENTEXTPROC    GLVertexArrayEXT::_glArrayElementEXT = 0;
PFNGLCOLORPOINTEREXTPROC    GLVertexArrayEXT::_glColorPointerEXT = 0;
PFNGLDRAWARRAYSEXTPROC      GLVertexArrayEXT::_glDrawArraysEXT = 0;
PFNGLEDGEFLAGPOINTEREXTPROC GLVertexArrayEXT::_glEdgeFlagPointerEXT = 0;
PFNGLGETPOINTERVEXTPROC     GLVertexArrayEXT::_glGetPointervEXT = 0;
PFNGLINDEXPOINTEREXTPROC    GLVertexArrayEXT::_glIndexPointerEXT = 0;
PFNGLNORMALPOINTEREXTPROC   GLVertexArrayEXT::_glNormalPointerEXT = 0;
PFNGLTEXCOORDPOINTEREXTPROC GLVertexArrayEXT::_glTexCoordPointerEXT = 0;
PFNGLVERTEXPOINTEREXTPROC   GLVertexArrayEXT::_glVertexPointerEXT = 0;

GLVertexArrayEXT::GLVertexArrayEXT( void):GLExtension( getName())
{
    GLEXTCAST _glArrayElementEXT    = GetExtensionProc("glArrayElementEXT");
    GLEXTCAST _glColorPointerEXT    = GetExtensionProc("glColorPointerEXT");
    GLEXTCAST _glDrawArraysEXT      = GetExtensionProc("glDrawArraysEXT");
    GLEXTCAST _glEdgeFlagPointerEXT = GetExtensionProc("glEdgeFlagPointerEXT");
    GLEXTCAST _glGetPointervEXT     = GetExtensionProc("glGetPointervEXT");
    GLEXTCAST _glIndexPointerEXT    = GetExtensionProc("glIndexPointerEXT");
    GLEXTCAST _glNormalPointerEXT   = GetExtensionProc("glNormalPointerEXT");
    GLEXTCAST _glTexCoordPointerEXT = GetExtensionProc("glTexCoordPointerEXT");
    GLEXTCAST _glVertexPointerEXT   = GetExtensionProc("glVertexPointerEXT");
}
