// Description:
//   Block View Base
//
// Copyright (C) 2003 Frank Becker
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation;  either version 2 of the License,  or (at your option) any  later
// version.
//
// This program is distributed in the hope that it will be useful,  but  WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
//
#include <Trace.hpp>

#include "BlockModel.hpp"
#include "BlockViewBase.hpp"

BlockViewBase::BlockViewBase( BlockModel &model):
    _model(model)
{
    XTRACE();
}

BlockViewBase::~BlockViewBase()
{
    XTRACE();
}

void BlockViewBase::update( void)
{
    //update any view state
}
